from django.db import connection, models
from django.db.models import Q
from django.db.models.expressions import RawSQL
from django.db.models.functions import Extract, ExtractYear
from django.utils.text import slugify

from .db import GroupedManager

import hashlib
import os


class Timetable(models.Model):
    name = models.CharField(max_length=64, unique=True, verbose_name="nom")
    url = models.URLField(max_length=255, unique=True, verbose_name="URL")
    slug = models.SlugField(max_length=64, unique=True, default="")

    def __str__(self):
        return self.name


    class Meta:
        verbose_name = "emploi du temps"
        verbose_name_plural = "emplois du temps"


class LastUpdate(models.Model):
    timetable = models.ForeignKey(Timetable, on_delete=models.CASCADE, verbose_name="emploi du temps")
    week = models.IntegerField(verbose_name="semaine")
    year = models.IntegerField(verbose_name="année")
    date = models.DateTimeField(verbose_name="date de mise à jour")


    class Meta:
        unique_together = ("timetable", "week", "year",)
        verbose_name = "dernière mise à jour"
        verbose_name_plural = "dernières mises à jour"


class Group(models.Model):
    name = models.CharField(max_length=255, verbose_name="nom")
    timetable = models.ForeignKey(Timetable, on_delete=models.CASCADE, verbose_name="emploi du temps")

    mention = models.CharField(max_length=32)
    subgroup = models.CharField(max_length=1, verbose_name="sous-groupe")
    td = models.IntegerField(verbose_name="groupe de TD", null=True)
    tp = models.IntegerField(verbose_name="groupe de TP", null=True)

    slug = models.SlugField(max_length=64, unique=True, default="")

    def corresponds_to(self, timetable_id, mention, subgroup, td, tp):
        return self.timetable.id == timetable_id and self.mention == mention and self.subgroup == subgroup and (self.td == td or self.td is None or td is None) and (self.tp == tp or self.tp is None or tp is None)

    def __str__(self):
        return self.name

    def save(self):
        try:
            parts = self.name.split("-")[-2:]
            group = parts[1].strip()[2:]

            self.mention = parts[0].strip()
            self.subgroup = group[0]

            if len(group) > 1:
                self.td = int(group[1])
            if len(group) > 2:
                self.tp = int(group[2])
        except:
            print("Malformed name: {0}. Ignoring".format(self.name))
        finally:
            self.slug = slugify(self.name)
            super(Group, self).save()


    class Meta:
        index_together = ("mention", "subgroup", "td", "tp",)
        unique_together = ("name", "timetable",)

        verbose_name = "groupe"
        verbose_name_plural = "groupes"


class Subscription(models.Model):
    email = models.EmailField(verbose_name="adresse")
    group = models.ForeignKey(Group, on_delete=models.CASCADE, verbose_name="groupe")
    active = models.BooleanField(verbose_name="activé", default=False, db_index=True)
    token = models.CharField(max_length=64, unique=True, default="")

    def save(self):
        if self.token == "":
            self.token = hashlib.sha1(os.urandom(128)).hexdigest()
        super(Subscription, self).save()


    class Meta:
        unique_together = ("email", "group",)
        verbose_name = "abonnement"
        verbose_name_plural = "abonnements"


class Room(models.Model):
    name = models.CharField(max_length=255, unique=True, verbose_name="nom")

    def __str__(self):
        return self.name


    class Meta:
        verbose_name = "salle"
        verbose_name_plural = "salles"


class CourseManager(GroupedManager):
    def __get_weeks(self, qs):
        extractYear = ExtractYear("begin")
        qs = qs.group_by("groups", "year", "week").order_by("groups__name", "year", "week")

        if connection.vendor == "postgresql":
            return qs.annotate(week=ExtractWeek("begin"), year=extractYear)
        else:
            return qs.annotate(week=RawSQL("""cast(strftime("%%W", "begin") as integer)""", []), year=extractYear)

    def get_courses_for_group(self, group, **filters):
        return self.get_queryset().filter(Q(groups__td__isnull=True) | Q(groups__td=group.td), Q(groups__tp__isnull=True) | Q(groups__tp=group.tp), groups__mention=group.mention, groups__subgroup=group.subgroup, **filters).order_by("begin")

    def get_weeks(self, **criteria):
        qs = self.get_queryset().filter(**criteria)
        return self.__get_weeks(qs)


class Course(models.Model):
    objects = CourseManager()

    name = models.CharField(max_length=255, verbose_name="nom", null=True)
    type_ = models.CharField(name="type", max_length=255, verbose_name="type de cours", null=True)
    timetable = models.ForeignKey(Timetable, on_delete=models.CASCADE, verbose_name="emploi du temps")
    notes = models.TextField(verbose_name="remarques", blank=True, null=True)

    groups = models.ManyToManyField(Group, verbose_name="groupes")
    rooms = models.ManyToManyField(Room, verbose_name="salles")

    begin = models.DateTimeField(verbose_name="début du cours", db_index=True)
    end = models.DateTimeField(verbose_name="fin du cours")

    def __str__(self):
        return self.name


    class Meta:
        verbose_name = "cours"
        verbose_name_plural = "cours"
